@@ -426,6 +426,9 @@ class TypeConverter(object):
         reference = pointer = array = False
 
         def add_type(templated_types):
+            if not name_tokens:
+                return
+
             # Partition tokens into name and modifier tokens.
             names = []
             modifiers = []
@@ -436,8 +439,6 @@ class TypeConverter(object):
                     names.append(t.name)
             name = ''.join(names)
 
-            assert_parse(name_tokens, 'not enough tokens')
-
             result.append(Type(name_tokens[0].start, name_tokens[-1].end,
                                name, templated_types, modifiers,
                                reference, pointer, array))
@@ -457,12 +458,15 @@ class TypeConverter(object):
                     elif tokens[new_end].name == '*':
                         pointer = True
                         new_end += 1
+                    elif tokens[new_end].name == '&':
+                        reference = True
+                        new_end += 1
                 add_type(self.to_type(new_tokens))
                 # If there is a comma after the template, we need to consume
                 # that here otherwise it becomes part of the name.
                 i = new_end
                 reference = pointer = array = False
-            elif token.name == ',':
+            elif token.name == ',' or token.name == '(':
                 add_type([])
                 reference = pointer = array = False
             elif token.name == '*':
@@ -471,7 +475,7 @@ class TypeConverter(object):
                 reference = True
             elif token.name == '[':
                 pointer = True
-            elif token.name == ']':
+            elif token.name == ']' or token.name == ')':
                 pass
             else:
                 name_tokens.append(token)
@@ -483,30 +487,33 @@ class TypeConverter(object):
         return result
 
     def declaration_to_parts(self, parts, needs_name_removed):
-        name = None
+        arrayBegin = 0
+        arrayEnd = 0
         default = []
+        other_tokens = []
+
+        # Handle default (initial) values properly.
+        for i, t in enumerate(parts):
+            if t.name == '[':
+                arrayBegin = i
+                other_tokens.append(t)
+            elif t.name == ']':
+                arrayEnd = i
+                other_tokens.append(t)
+            elif t.name == '=':
+                default = parts[i + 1:]
+                parts = parts[:i]
+                break
+
+        if arrayBegin < arrayEnd:
+            parts = parts[:arrayBegin] + parts[arrayEnd + 1:]
+
+        name = None
         if needs_name_removed:
-            # Handle default (initial) values properly.
-            for i, t in enumerate(parts):
-                if t.name == '=':
-                    default = parts[i + 1:]
-                    name = parts[i - 1].name
-                    if name == ']' and parts[i - 2].name == '[':
-                        name = parts[i - 3].name
-                        i -= 1
-                    parts = parts[:i - 1]
-                    break
-            else:
-                if parts[-1].token_type == tokenize.NAME:
-                    name = parts.pop().name
-                else:
-                    # TODO(nnorwitz): this is a hack that happens for code like
-                    # Register(Foo<T>); where it thinks this is a function call
-                    # but it's actually a declaration.
-                    name = '???'
+            name = parts.pop().name
+
         modifiers = []
         type_name = []
-        other_tokens = []
         templated_types = []
         i = 0
         end = len(parts)
@@ -523,9 +530,6 @@ class TypeConverter(object):
                 next_index = i + 1
                 if next_index < end and parts[next_index].name == '::':
                     i += 1
-            elif p.name in ('[', ']', '='):
-                # These are handled elsewhere.
-                other_tokens.append(p)
             elif p.name not in ('*', '&', '>'):
                 # Ensure that names have a space between them.
                 if (type_name and type_name[-1].token_type == tokenize.NAME and
